@@ -16,7 +16,6 @@
 #include "Store.h"
 
 #include <climits>
-#include <cstddef>
 
 #if USE_CBDATA_DEBUG
 #include <algorithm>
@@ -47,6 +46,8 @@ public:
 
 #endif
 
+#define OFFSET_OF(TYPE, MEMBER) ((size_t) &(((TYPE) *)0)->(MEMBER))
+
 /**
  * Manage a set of registered callback data pointers.
  * One of the easiest ways to make Squid coredump is to issue a
@@ -70,7 +71,7 @@ public:
     MEMPROXY_CLASS(cbdata);
 #endif
 
-    /** \todo examine making cbdata templated on this - so we get type
+    /* TODO: examine making cbdata templated on this - so we get type
      * safe access to data - RBC 20030902 */
 public:
 #if USE_CBDATA_DEBUG
@@ -90,8 +91,6 @@ public:
     {}
     ~cbdata();
 
-    static cbdata *FromUserData(const void *);
-
     int valid;
     int32_t locks;
     cbdata_type type;
@@ -117,14 +116,25 @@ public:
 
 #if !WITH_VALGRIND
     size_t dataSize() const { return sizeof(data);}
+    static long MakeOffset();
+    static const long Offset;
 #endif
     /* MUST be the last per-instance member */
     void *data;
 };
 
-static_assert(std::is_standard_layout<cbdata>::value, "the behavior of offsetof(cbdata) is defined");
-
 const long cbdata::Cookie((long)0xDEADBEEF);
+#if !WITH_VALGRIND
+const long cbdata::Offset(MakeOffset());
+
+long
+cbdata::MakeOffset()
+{
+    cbdata *zero = (cbdata *)0L;
+    void **dataOffset = &zero->data;
+    return (long)dataOffset;
+}
+#endif
 
 static OBJH cbdataDump;
 #if USE_CBDATA_DEBUG
@@ -161,16 +171,6 @@ cbdata::~cbdata()
     cbdata_index[type].pool->freeOne(p);
 }
 
-cbdata *
-cbdata::FromUserData(const void *p) {
-#if WITH_VALGRIND
-    return cbdata_htable.at(p);
-#else
-    const auto t = static_cast<const char *>(p) - offsetof(cbdata, data);
-    return reinterpret_cast<cbdata *>(const_cast<char *>(t));
-#endif
-}
-
 static void
 cbdataInternalInitType(cbdata_type type, const char *name, int size)
 {
@@ -186,7 +186,8 @@ cbdataInternalInitType(cbdata_type type, const char *name, int size)
     snprintf(label, strlen(name) + 20, "cbdata %s (%d)", name, (int) type);
 
 #if !WITH_VALGRIND
-    size += offsetof(cbdata, data);
+    assert((size_t)cbdata::Offset == (sizeof(cbdata) - ((cbdata *)NULL)->dataSize()));
+    size += cbdata::Offset;
 #endif
 
     cbdata_index[type].pool = memPoolCreate(label, size);
@@ -296,7 +297,12 @@ cbdataRealFree(cbdata *c, const char *file, const int line)
 void *
 cbdataInternalFree(void *p, const char *file, int line)
 {
-    auto *c = cbdata::FromUserData(p);
+    cbdata *c;
+#if WITH_VALGRIND
+    c = cbdata_htable.at(p);
+#else
+    c = (cbdata *) (((char *) p) - cbdata::Offset);
+#endif
 #if USE_CBDATA_DEBUG
     debugs(45, 3, p << " " << file << ":" << line);
 #else
@@ -327,10 +333,16 @@ cbdataInternalLockDbg(const void *p, const char *file, int line)
 cbdataInternalLock(const void *p)
 #endif
 {
+    cbdata *c;
+
     if (p == NULL)
         return;
 
-    auto *c = cbdata::FromUserData(p);
+#if WITH_VALGRIND
+    c = cbdata_htable.at(p);
+#else
+    c = (cbdata *) (((char *) p) - cbdata::Offset);
+#endif
 
 #if USE_CBDATA_DEBUG
     debugs(45, 3, p << "=" << (c ? c->locks + 1 : -1) << " " << file << ":" << line);
@@ -353,10 +365,16 @@ cbdataInternalUnlockDbg(const void *p, const char *file, int line)
 cbdataInternalUnlock(const void *p)
 #endif
 {
+    cbdata *c;
+
     if (p == NULL)
         return;
 
-    auto *c = cbdata::FromUserData(p);
+#if WITH_VALGRIND
+    c = cbdata_htable.at(p);
+#else
+    c = (cbdata *) (((char *) p) - cbdata::Offset);
+#endif
 
 #if USE_CBDATA_DEBUG
     debugs(45, 3, p << "=" << (c ? c->locks - 1 : -1) << " " << file << ":" << line);
@@ -393,12 +411,18 @@ cbdataInternalUnlock(const void *p)
 int
 cbdataReferenceValid(const void *p)
 {
+    cbdata *c;
+
     if (p == NULL)
         return 1;       /* A NULL pointer cannot become invalid */
 
     debugs(45, 9, p);
 
-    const auto c = cbdata::FromUserData(p);
+#if WITH_VALGRIND
+    c = cbdata_htable.at(p);
+#else
+    c = (cbdata *) (((char *) p) - cbdata::Offset);
+#endif
 
     c->check(__LINE__);
 
@@ -478,7 +502,7 @@ cbdataDump(StoreEntry * sentry)
 #if WITH_VALGRIND
             int obj_size = pool->objectSize();
 #else
-            int obj_size = pool->objectSize() - offsetof(cbdata, data);
+            int obj_size = pool->objectSize() - cbdata::Offset;
 #endif
             storeAppendPrintf(sentry, "%s\t%d\t%ld\t%ld\n", pool->objectType() + 7, obj_size, (long int)pool->getMeter().inuse.currentLevel(), (long int)obj_size * pool->getMeter().inuse.currentLevel());
         }
@@ -492,6 +516,17 @@ cbdataDump(StoreEntry * sentry)
     storeAppendPrintf(sentry, "\nsee also \"Memory utilization\" for detailed per type statistics\n");
 }
 
+CallbackData &
+CallbackData::operator =(const CallbackData &other)
+{
+    if (data_ != other.data_) { // assignment to self and no-op assignments
+        auto old = data_;
+        data_ = cbdataReference(other.data_);
+        cbdataReferenceDone(old);
+    }
+    return *this;
+}
+
 CBDATA_CLASS_INIT(generic_cbdata);
 
 #if USE_CBDATA_DEBUG
